/* ****************************************************************** **
**    OpenSees - Open System for Earthquake Engineering Simulation    **
**          Pacific Earthquake Engineering Research Center            **
**                                                                    **
**                                                                    **
** (C) Copyright 1999, The Regents of the University of California    **
** All Rights Reserved.                                               **
**                                                                    **
** Commercial use of this program without express permission of the   **
** University of California, Berkeley, is strictly prohibited.  See   **
** file 'COPYRIGHT'  in main directory for information on usage and   **
** redistribution,  and for a DISCLAIMER OF ALL WARRANTIES.           **
**                                                                    **
** Developed by:                                                      **
**   Frank McKenna (fmckenna@ce.berkeley.edu)                         **
**   Gregory L. Fenves (fenves@ce.berkeley.edu)                       **
**   Filip C. Filippou (filippou@ce.berkeley.edu)                     **
**                                                                    **
** ****************************************************************** */

// $Revision: 1.2 $
// $Date: 2003-06-10 00:36:09 $
// $Source: /usr/local/cvs/OpenSees/SRC/element/forceBeamColumn/UserDefinedBeamIntegration.h,v $

#ifndef UserDefinedBeamIntegration_h
#define UserDefinedBeamIntegration_h

#include "BeamIntegration.h"

#include <Vector.h>

class Channel;
class FEM_ObjectBroker;

class UserDefinedBeamIntegration : public BeamIntegration
{
 public:
  UserDefinedBeamIntegration(int nIP, const Vector &pt, const Vector &wt);
  UserDefinedBeamIntegration();
  ~UserDefinedBeamIntegration();
  
  void getSectionLocations(int numSections, double L, double *xi) const;
  void getSectionWeights(int numSections, double L, double *wt) const;

  BeamIntegration *getCopy(void);

  int sendSelf(int cTag, Channel &theChannel);
  int recvSelf(int cTag, Channel &theChannel, FEM_ObjectBroker &theBroker);

  int setParameter(const char **argv, int argc, Parameter &param);
  int updateParameter(int parameterID, Information &info);

  void Print(OPS_Stream &s, int flag = 0);  

 private:
  Vector pts;
  Vector wts;
};

#endif
