/* ****************************************************************** **
**    OpenSees - Open System for Earthquake Engineering Simulation    **
**          Pacific Earthquake Engineering Research Center            **
**                                                                    **
**                                                                    **
** (C) Copyright 1999, The Regents of the University of California    **
** All Rights Reserved.                                               **
**                                                                    **
** Commercial use of this program without express permission of the   **
** University of California, Berkeley, is strictly prohibited.  See   **
** file 'COPYRIGHT'  in main directory for information on usage and   **
** redistribution,  and for a DISCLAIMER OF ALL WARRANTIES.           **
**                                                                    **
** Developed by:                                                      **
**   Frank McKenna (fmckenna@ce.berkeley.edu)                         **
**   Gregory L. Fenves (fenves@ce.berkeley.edu)                       **
**   Filip C. Filippou (filippou@ce.berkeley.edu)                     **
**                                                                    **
** ****************************************************************** */

// $Revision: 1.1 $
// $Date: 2006-01-18 21:58:24 $
// $Source: /usr/local/cvs/OpenSees/SRC/element/forceBeamColumn/UserDefinedHingeIntegration.h,v $

#ifndef UserDefinedHingeIntegration_h
#define UserDefinedHingeIntegration_h

#include "BeamIntegration.h"

#include <Vector.h>

class Channel;
class FEM_ObjectBroker;

class UserDefinedHingeIntegration : public BeamIntegration
{
 public:
  UserDefinedHingeIntegration(int npL, const Vector &ptL, const Vector &wtL,
			      int npR, const Vector &ptR, const Vector &wtR);
  UserDefinedHingeIntegration();
  ~UserDefinedHingeIntegration();
  
  void getSectionLocations(int numSections, double L, double *xi) const;
  void getSectionWeights(int numSections, double L, double *wt) const;

  BeamIntegration *getCopy(void);

  int sendSelf(int cTag, Channel &theChannel);
  int recvSelf(int cTag, Channel &theChannel, FEM_ObjectBroker &theBroker);

  void Print(OPS_Stream &s, int flag = 0);

 private:
  Vector ptsL;
  Vector wtsL;
  Vector ptsR;
  Vector wtsR;
};

#endif
